#include <Application.h>
#include <MessageRunner.h>
#include <stdio.h>
#include <stdlib.h>
#include <OS.h>

#include <private/shared/AutoDeleter.h>

enum {
	second = 1000000,
};

enum {
	timerMsg = 1,
};

enum {
	maxDelay = second/10,
	minDelay = 1,
	stepCnt = 5000,
};

class TestApplication: public BApplication
{
private:
	ObjectDeleter<BMessageRunner> fTimer;
	bigtime_t fScheduledTime;

	int32 fCurStep;
	bigtime_t fDelays[stepCnt];
	bigtime_t fStat[stepCnt];

public:
	TestApplication(): BApplication("application/x-vnd.test-app"),
		fCurStep(0)
	{
		for (int32 i = 0; i < stepCnt; i++) {
			fDelays[i] = minDelay + (maxDelay - minDelay)*i/(stepCnt - 1);
		}
	}

	void Schedule(bigtime_t time)
	{
		bigtime_t now = system_time();
		fScheduledTime = time;
		fTimer.SetTo(new BMessageRunner(be_app_messenger, new BMessage(timerMsg), time - now, 1));
	}

	void Step()
	{
		bigtime_t now = system_time();
		Schedule(now + fDelays[fCurStep]);
	}

	void ReadyToRun()
	{
		Step();
	}

	void MessageReceived(BMessage *msg)
	{
		switch (msg->what) {
		case timerMsg: {
			bigtime_t now = system_time();
			fStat[fCurStep] = now - fScheduledTime;
			fCurStep++;
			if (fCurStep < stepCnt) {
				Step();
			} else {
				for (int32 i = 0; i < stepCnt; i++) {
					printf("%.06E %.06E\n", double(fDelays[i])/second, double(fStat[i])/second);
				}
				PostMessage(B_QUIT_REQUESTED);
			}
			break;
		}
		default:
			BApplication::MessageReceived(msg);
		}
	}
};


int main()
{
	TestApplication app;
	app.Run();
	return 0;
}
