#include <Application.h>
#include <MessageRunner.h>
#include <stdio.h>
#include <stdlib.h>
#include <OS.h>

#include <private/shared/AutoDeleter.h>

enum {
	second = 1000000,
};

enum {
	timerMsg = 1,
};

enum {
	maxDelay = second/10,
	minDelay = 1,
	stepCnt = 5000,
};

class TestApplication: public BApplication
{
private:
	bigtime_t fScheduledTime;

	int32 fCurStep;
	bigtime_t fDelays[stepCnt];
	bigtime_t fStat[stepCnt];

public:
	TestApplication(): BApplication("application/x-vnd.test-app"),
		fCurStep(0)
	{
		for (int32 i = 0; i < stepCnt; i++) {
			fDelays[i] = minDelay + (maxDelay - minDelay)*i/(stepCnt - 1);
		}
	}

	void ReadyToRun()
	{
		while (fCurStep < stepCnt) {
			{
				bigtime_t now = system_time();
				fScheduledTime = now + fDelays[fCurStep];
				snooze_until(fScheduledTime, B_SYSTEM_TIMEBASE);
			}
			{
				bigtime_t now = system_time();
				fStat[fCurStep] = now - fScheduledTime;
			}
			fCurStep++;
		}
		for (int32 i = 0; i < stepCnt; i++) {
			printf("%.06E %.06E\n", double(fDelays[i])/second, double(fStat[i])/second);
		}
		PostMessage(B_QUIT_REQUESTED);
	}

};


int main()
{
	TestApplication app;
	app.Run();
	return 0;
}
