#include "BTimer.hpp"

void BTimer::OnInterval(void) {
	//dummy
}

int32 timer_thread_handler(void *data) {
	BTimer *timer = (BTimer*)data;
	timer->ThreadHandler();
	return 0;
}

void BTimer::ThreadHandler(void) {
	bigtime_t interval;
	bool running;
	
	for (;;) {
		acquire_sem(lock_sem);
		interval = this->interval;
		running = this->running;
		release_sem(lock_sem);
		
		if (!running)
			acquire_sem(pause_sem);
		
		acquire_sem(lock_sem);
		interval = this->interval;
		running = this->running;
		release_sem(lock_sem);
		
		if (!running)
			break;
		
		OnInterval();
		
		snooze(interval);
	}
	
	delete_sem(pause_sem);
	delete_sem(lock_sem);
}

BTimer::BTimer(bigtime_t usec) {
	interval = usec;
	running = false;
	pause_sem = create_sem(0, "pause_sem");
	lock_sem = create_sem(1, "lock_sem");
	
	head = NULL;
	prev = this;
	next = this;
	
	timer_thread = spawn_thread(timer_thread_handler,
		"timer_thread", B_DISPLAY_PRIORITY,
		this);
	resume_thread(timer_thread);
}

BTimer::~BTimer() {
	if (Running())
		Pause();
	
	//release the semaphore without setting running to true, thus indicating
	//that the thread handler should quit
	release_sem(pause_sem);
	
	prev->next = next;
	next->prev = prev;
	if (head && *head == this) {
		*head = next;
		if (next == this)
			*head = NULL;
	}
}

void BTimer::Resume(void) {
	bool r = Running();
	
	if (r)
		return;
	
	acquire_sem(lock_sem);
	running = 1;
	release_sem(lock_sem);
	
	//tell the thread handler it can go ahead
	release_sem(pause_sem);
}

void BTimer::Pause(void) {
	bool r = Running();
	
	if (!r)
		return;
	
	acquire_sem(lock_sem);
	running = 0;
	release_sem(lock_sem);
}

bool BTimer::Running(void) {
	bool ret;
	
	acquire_sem(lock_sem);
	ret = running;
	release_sem(lock_sem);
	
	return ret;
}

void BTimer::SetInterval(bigtime_t usec) {
	acquire_sem(lock_sem);
	interval = usec;
	release_sem(lock_sem);
}

bigtime_t BTimer::Interval(void) {
	bigtime_t ret;
	
	acquire_sem(lock_sem);
	ret = interval;
	release_sem(lock_sem);
	
	return ret;
}
